/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "view/edit-view.h"
#include "utils//ui-utils.h"
#include "utils/logger.h"
#include "utils/config.h"
#include "utils/event-record.h"
#include "view/recurrence-view.h"
#include "utils/settings-utils.h"

#include <time.h>

static const char *save_button          = "Save";
static const char *label_title          = "Subject";
static const char *subject_guide        = "Enter the subject";
static const char *label_from           = "From";
static const char *label_to             = "To";
static const char *label_time_zone      = "TimeZone";
static const char *label_all_day        = "All day";
static const char *label_location       = "Location";
static const char *location_guide       = "Enter the location";
static const char *label_description    = "Description";
static const char *description_guide    = "Enter the description";
static const char *label_reminder       = "Reminder";
static const char *label_recurrence     = "Recurrence";
static const char *label_priority       = "Priority";
static const char *label_sensitivity    = "Sensitivity";
static const char *label_status         = "Status";
static const char *label_none           = "None";
static char *button_cancel              = "Cancel";
static char *button_set                 = "Set";

typedef struct
{
    window_obj          *win;
    Evas_Object         *navi_frame;
    Elm_Object_Item     *naviframe_item;

    Evas_Object         *layout;
    Evas_Object         *genlist;

    Evas_Object         *start_datetime;
    Evas_Object         *end_datetime;
    Evas_Object         *start_date_btn;
    Evas_Object         *start_time_btn;
    Evas_Object         *end_date_btn;
    Evas_Object         *end_time_btn;
    Evas_Object         *check_object;
    Evas_Object         *priority_button;
    Evas_Object         *reminder_btn;
    Evas_Object         *reminder_ctxpopup;
    calendar_alarm_time_unit_type_e       reminder_type;
    int                 reminder;
    char                *reminder_type_name;
    Evas_Object         *sensitivity_button;
    Evas_Object         *status_button;
    Evas_Object         *context_popup;
    Evas_Object         *recurrence_btn;
    Evas_Object         *popup;

    char                *title;
    char                *description;
    char                *location;

    calendar_event_priority_e priority;
    calendar_sensitivity_e sensitivity;
    calendar_event_status_e status;

    Elm_Object_Item     *genlist_from_item;
    Elm_Object_Item     *genlist_to_item;
    Elm_Object_Item     *genlist_reminder_item;
    Elm_Object_Item     *genlist_location_item;
    Elm_Object_Item     *genlist_description_item;

    calendar_record_h   event_record;
    recurrence_repeat_data recurrence_data;

    Evas_Object         *date_time_popup;

    struct tm           stm; // start
    struct tm           etm; // end

    Eina_Bool           is_all_day;
} edit_view_data;

static bool _close_active_popup(edit_view_data *view_data);

static void _scroll_genlist_top_cb(void *data, Evas_Object *obj, void *event_info)
{
    edit_view_data *ad = (edit_view_data *)data;
    RETM_IF(!ad, "Passed NULL input data");

    if(ad->genlist_location_item == (Elm_Object_Item *)event_info
       || ad->genlist_description_item == (Elm_Object_Item *)event_info
       || ad->genlist_reminder_item == (Elm_Object_Item *)event_info)
    {
        elm_genlist_item_show((Elm_Object_Item *)event_info, ELM_GENLIST_ITEM_SCROLLTO_MIDDLE);
    }
}

static void _location_entry_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    if(view_data->location != NULL)
    {
        free(view_data->location);
        view_data->location = NULL;
    }
    view_data->location = strdup(elm_entry_entry_get(obj));
}

static void _description_entry_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    if(view_data->description != NULL)
    {
        free(view_data->description);
        view_data->description = NULL;
    }
    view_data->description = strdup(elm_entry_entry_get(obj));
}

static void _title_entry_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    if(view_data->title != NULL)
    {
        free(view_data->title);
        view_data->title = NULL;
    }
    view_data->title = strdup(elm_entry_entry_get(obj));
}

static Evas_Object *_get_title_content(void *data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!data, NULL, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    Evas_Object *box = elm_box_add(obj);
    elm_box_horizontal_set(box, EINA_TRUE);
    elm_box_align_set(box, 0.0, 0.5);
    elm_box_padding_set(box, BOX_PADDING, 0);
    evas_object_show(box);

    Evas_Object *label = elm_label_add(box);
    elm_object_text_set(label, label_title);
    evas_object_show(label);

    Evas_Object *entry = elm_entry_add(box);
    elm_entry_single_line_set(entry, EINA_TRUE);
    elm_entry_scrollable_set(entry, EINA_TRUE);
    elm_scroller_policy_set(entry, ELM_SCROLLER_POLICY_AUTO, ELM_SCROLLER_POLICY_OFF);
    Elm_Entry_Filter_Limit_Size limit = {SUBJECT_CHAR_LIMIT, SUBJECT_BYTE_LIMIT};
    elm_entry_markup_filter_append(entry, elm_entry_filter_limit_size, &limit);
    elm_entry_cnp_mode_set(entry, ELM_CNP_MODE_PLAINTEXT);

    evas_object_size_hint_weight_set(entry, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(entry, EVAS_HINT_FILL, EVAS_HINT_FILL);

    elm_object_part_text_set(entry, "elm.guide", subject_guide);
    evas_object_smart_callback_add(entry, "changed", _title_entry_changed_cb, view_data);
    evas_object_show(entry);

    if (view_data->title == NULL)
    {
        char *summary = NULL;
        calendar_record_get_str_p(view_data->event_record, _calendar_event.summary, &summary);
        if(summary)
        {
            view_data->title = strdup(summary);
        }
    }

    elm_entry_entry_set(entry, view_data->title);

    elm_box_pack_end(box, label);
    elm_box_pack_end(box, entry);

    return box;
}

static char *_get_label_from(void *data, Evas_Object *obj, const char *part)
{
    return strdup(label_from);
}

static void _on_datetime_popup_cancel_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "data is null");
    edit_view_data *view = data;
    _close_active_popup(view);
}

static void _end_date_changed(void *data, Evas_Object* obj)
{
    RETM_IF(!data, "Passed NULL input data");
    edit_view_data *ad = (edit_view_data *)data;
    elm_datetime_value_get(ad->end_datetime, &ad->etm);
    _close_active_popup(ad);
}

static void _start_date_changed(edit_view_data *view, Evas_Object* obj)
{
    elm_datetime_value_get(obj, &view->stm);
    _close_active_popup(view);
}

static void _on_datetime_popup_set_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "data is null");
    edit_view_data *view = data;

    Elm_Object_Item *selected_item = elm_genlist_selected_item_get(view->genlist);

    if(selected_item == view->genlist_from_item)
    {
        _start_date_changed(view, view->start_datetime);
    }
    else if(selected_item == view->genlist_to_item)
    {
        _end_date_changed(view, view->end_datetime);
    }
    elm_genlist_item_update(selected_item);
}

static Evas_Object *_date_time_popup_add(Evas_Object *parent, edit_view_data *view,
                                         Evas_Smart_Cb set_cb, Evas_Smart_Cb cancel_cb)
{
    Evas_Object *popup = elm_popup_add(parent);
    view->date_time_popup = popup;

    elm_popup_orient_set(popup, ELM_POPUP_ORIENT_CENTER);
    evas_object_show(popup);

    Evas_Object *cancel_btn = elm_button_add(popup);
    elm_object_style_set(cancel_btn, "popup");
    elm_object_text_set(cancel_btn, button_cancel);
    elm_object_part_content_set(popup, "button1", cancel_btn);
    evas_object_smart_callback_add(cancel_btn, "clicked", cancel_cb, view);

    Evas_Object *set_btn = elm_button_add(popup);
    elm_object_style_set(set_btn, "popup");
    elm_object_text_set(set_btn, button_set);
    elm_object_part_content_set(popup, "button2", set_btn);
    evas_object_smart_callback_add(set_btn, "clicked", set_cb, view);

    return popup;
}

static void _get_start_datetime(edit_view_data *view, Evas_Object *parent, const char *format)
{
    if(!view->start_datetime)
    {
        view->start_datetime = elm_datetime_add(parent);
        evas_object_size_hint_weight_set(view->start_datetime, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_size_hint_align_set(view->start_datetime, EVAS_HINT_FILL, EVAS_HINT_FILL);
    }

    elm_datetime_value_set(view->start_datetime, &view->stm);
    elm_datetime_format_set(view->start_datetime, format);
    evas_object_show(view->start_datetime);
}

static void _get_end_datetime(edit_view_data *view, Evas_Object *parent, const char *format)
{
    if(!view->end_datetime)
    {
        view->end_datetime = elm_datetime_add(parent);
        evas_object_size_hint_weight_set(view->end_datetime, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_size_hint_align_set(view->end_datetime, EVAS_HINT_FILL, EVAS_HINT_FILL);
    }

    elm_datetime_value_set(view->end_datetime, &view->etm);
    elm_datetime_format_set(view->end_datetime, format);
    evas_object_show(view->end_datetime);
}

static void _start_date_btn_cb(void *data, Evas_Object* obj, void *event_info)
{
    RETM_IF(!data, "data is null");
    edit_view_data *view = (edit_view_data *)data;
    Evas_Object *popup = _date_time_popup_add(view->win->win, view, _on_datetime_popup_set_cb, _on_datetime_popup_cancel_cb);
    evas_object_show(popup);

    _get_start_datetime(view, popup, "%d %b %Y");
    elm_object_content_set(popup, view->start_datetime);
}

static void _start_time_btn_cb(void *data, Evas_Object* obj, void *event_info)
{
    RETM_IF(!data, "data is null");
    edit_view_data *view = (edit_view_data *)data;
    Evas_Object *popup = _date_time_popup_add(view->win->win, view, _on_datetime_popup_set_cb, _on_datetime_popup_cancel_cb);
    evas_object_show(popup);

    _get_start_datetime(view, popup, "%R");
    elm_object_content_set(popup, view->start_datetime);
}

static void _end_date_btn_cb(void *data, Evas_Object* obj, void *event_info)
{
    RETM_IF(!data, "data is null");
    edit_view_data *view = (edit_view_data *)data;

    Evas_Object *popup = _date_time_popup_add(view->win->win, view, _on_datetime_popup_set_cb, _on_datetime_popup_cancel_cb);
    evas_object_show(popup);

    _get_end_datetime(view, popup, "%d %b %Y");

    elm_object_content_set(popup, view->end_datetime);
}

static void _end_time_btn_cb(void *data, Evas_Object* obj, void *event_info)
{
    RETM_IF(!data, "data is null");
    edit_view_data *view = (edit_view_data *)data;

    Evas_Object *popup = _date_time_popup_add(view->win->win, view, _on_datetime_popup_set_cb, _on_datetime_popup_cancel_cb);
    evas_object_show(popup);

    _get_end_datetime(view, popup, "%R");

    elm_object_content_set(popup, view->end_datetime);
}

static Evas_Object *_get_content_from(void *item_data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!item_data, NULL, "Passed NULL input data");
    edit_view_data *data = (edit_view_data *)item_data;

    if(strcmp(part, "elm.swallow.end") == 0)
    {
        Evas_Object *box = elm_box_add(obj);
        elm_box_horizontal_set(box, EINA_TRUE);
        evas_object_show(box);

        data->start_date_btn = elm_button_add(box);
        evas_object_show(data->start_date_btn);
        evas_object_smart_callback_add(data->start_date_btn, "clicked", _start_date_btn_cb, data);

        char buf[DATE_TIME_BUF_LEN] = {0};
        strftime(buf, DATE_TIME_BUF_LEN, "%x", &data->stm);
        elm_object_text_set(data->start_date_btn, buf);
        evas_object_size_hint_weight_set(data->start_date_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_size_hint_min_set(data->start_date_btn, ELM_SCALE_SIZE(BUTTON_SCALE_DATETIME_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));
        elm_box_pack_end(box, data->start_date_btn);

        if(!data->is_all_day)
        {
            data->start_time_btn = elm_button_add(box);
            evas_object_show(data->start_time_btn);
            evas_object_smart_callback_add(data->start_time_btn, "clicked", _start_time_btn_cb, data);

            strftime(buf, DATE_TIME_BUF_LEN, "%R", &data->stm);
            elm_object_text_set(data->start_time_btn, buf);
            evas_object_size_hint_weight_set(data->start_time_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
            evas_object_size_hint_min_set(data->start_time_btn, ELM_SCALE_SIZE(BUTTON_SCALE_DATETIME_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));
            elm_box_pack_end(box, data->start_time_btn);
        }

        return box;
    }

    return NULL;
}

static char *_get_label_to(void *data, Evas_Object *obj, const char *part)
{
    return strdup(label_to);
}

static Evas_Object *_get_content_to(void *item_data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!item_data, NULL, "Passed NULL input data");
    edit_view_data *data = (edit_view_data *)item_data;

    if(strcmp(part, "elm.swallow.end") == 0)
    {
        Evas_Object *box = elm_box_add(obj);
        elm_box_horizontal_set(box, EINA_TRUE);
        evas_object_show(box);

        data->end_date_btn = elm_button_add(box);
        evas_object_show(data->end_date_btn);
        evas_object_smart_callback_add(data->end_date_btn, "clicked", _end_date_btn_cb, data);

        char buf[DATE_TIME_BUF_LEN] = {0};
        strftime(buf, DATE_TIME_BUF_LEN, "%x", &data->etm);
        elm_object_text_set(data->end_date_btn, buf);
        evas_object_size_hint_weight_set(data->end_date_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_size_hint_min_set(data->end_date_btn, ELM_SCALE_SIZE(BUTTON_SCALE_DATETIME_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));

        elm_box_pack_end(box, data->end_date_btn);

        data->end_datetime = elm_datetime_add(data->layout);
        evas_object_size_hint_weight_set(data->end_datetime, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_size_hint_align_set(data->end_datetime, EVAS_HINT_FILL, EVAS_HINT_FILL);

        if(!data->is_all_day)
        {
            data->end_time_btn = elm_button_add(box);
            evas_object_show(data->end_time_btn);
            evas_object_smart_callback_add(data->end_time_btn, "clicked", _end_time_btn_cb, data);
            strftime(buf, DATE_TIME_BUF_LEN, "%R", &data->etm);
            elm_object_text_set(data->end_time_btn, buf);
            evas_object_size_hint_weight_set(data->end_time_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
            evas_object_size_hint_min_set(data->end_time_btn, ELM_SCALE_SIZE(BUTTON_SCALE_DATETIME_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));
            elm_box_pack_end(box, data->end_time_btn);
        }

        return box;
    }

    return NULL;
}

static char *_label_timezone_get(void *data, Evas_Object *obj, const char *part)
{
    return strdup(label_time_zone);
}

static Evas_Object *_get_content_timezone(void *data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!data, NULL, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;
    if(strcmp(part, "elm.swallow.end") != 0)
    {
        return NULL;
    }

    Evas_Object *label = elm_label_add(view_data->layout);

    char *timezone = NULL;
    calendar_record_get_str(view_data->event_record, _calendar_event.start_tzid, &timezone);
    elm_object_text_set(label, timezone);
    free(timezone);

    evas_object_size_hint_weight_set(label, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(label, EVAS_HINT_FILL, EVAS_HINT_FILL);

    return label;
}

static char *_get_label_allday(void *data, Evas_Object *obj, const char *part)
{
    return strdup(label_all_day);
}

static void __allday_changed(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    edit_view_data *ad = (edit_view_data *)data;

    ad->is_all_day = !ad->is_all_day;

    elm_genlist_item_update(ad->genlist_from_item);
    elm_genlist_item_update(ad->genlist_to_item);
}

static Evas_Object *_get_content_allday(void *data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!data, NULL, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    if(strcmp(part, "elm.swallow.end") != 0)
    {
        return NULL;
    }

    view_data->check_object = elm_check_add(view_data->layout);

    int is_all_day = 0;
    calendar_record_get_int(view_data->event_record, _calendar_event.is_allday, &is_all_day);

    view_data->is_all_day = is_all_day;
    elm_object_style_set(view_data->check_object, "on&off");
    elm_check_state_set(view_data->check_object, is_all_day);
    evas_object_propagate_events_set(view_data->check_object, EINA_FALSE);
    evas_object_smart_callback_add(view_data->check_object, "changed", __allday_changed, view_data);

    return view_data->check_object;
}

static Evas_Object *_get_content_location(void *data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!data, NULL, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data  *)data;
    Evas_Object *box = elm_box_add(obj);
    elm_box_horizontal_set(box, EINA_TRUE);
    elm_box_align_set(box, 0.0, 0.5);
    elm_box_padding_set(box, BOX_PADDING, 0);
    evas_object_show(box);

    Evas_Object *label = elm_label_add(box);
    elm_object_text_set(label, label_location);
    evas_object_show(label);

    Evas_Object *entry = elm_entry_add(box);
    elm_entry_single_line_set(entry, EINA_TRUE);
    elm_entry_scrollable_set(entry, EINA_TRUE);
    elm_scroller_policy_set(entry, ELM_SCROLLER_POLICY_AUTO, ELM_SCROLLER_POLICY_OFF);
    elm_entry_cnp_mode_set(entry, ELM_CNP_MODE_PLAINTEXT);

    evas_object_size_hint_weight_set(entry, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(entry, EVAS_HINT_FILL, EVAS_HINT_FILL);

    elm_object_part_text_set(entry, "elm.guide", location_guide);
    evas_object_smart_callback_add(entry, "changed", _location_entry_changed_cb, view_data);
    evas_object_show(entry);

    if (view_data->location == NULL)
    {
        char *location = NULL;
        calendar_record_get_str_p(view_data->event_record, _calendar_event.location, &location);
        if(location)
        {
            view_data->location = strdup(location);
        }
    }
    elm_entry_entry_set(entry, view_data->location);

    elm_box_pack_end(box, label);
    elm_box_pack_end(box, entry);

    return box;
}

static Evas_Object *_get_content_description(void *data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!data, NULL, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data  *)data;
    Evas_Object *box = elm_box_add(obj);
    elm_box_horizontal_set(box, EINA_TRUE);
    elm_box_align_set(box, 0.0, 0.5);
    elm_box_padding_set(box, BOX_PADDING, 0);
    evas_object_show(box);

    Evas_Object *label = elm_label_add(box);
    elm_object_text_set(label, label_description);
    evas_object_show(label);

    Evas_Object *entry = elm_entry_add(box);
    elm_entry_single_line_set(entry, EINA_TRUE);
    elm_entry_scrollable_set(entry, EINA_TRUE);
    elm_scroller_policy_set(entry, ELM_SCROLLER_POLICY_AUTO, ELM_SCROLLER_POLICY_OFF);
    elm_entry_cnp_mode_set(entry, ELM_CNP_MODE_PLAINTEXT);

    evas_object_size_hint_weight_set(entry, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(entry, EVAS_HINT_FILL, EVAS_HINT_FILL);

    elm_object_part_text_set(entry, "elm.guide", description_guide);
    evas_object_smart_callback_add(entry, "changed", _description_entry_changed_cb, view_data);
    evas_object_show(entry);

    if (view_data->description == NULL)
    {
        char *description = NULL;
        calendar_record_get_str_p(view_data->event_record, _calendar_event.description, &description);
        if(description)
        {
            view_data->description = strdup(description);
        }
    }
    elm_entry_entry_set(entry, view_data->description);

    elm_box_pack_end(box, label);
    elm_box_pack_end(box, entry);

    return box;
}

static char *_get_label_reminder(void *data, Evas_Object *obj, const char *part)
{
    if(strcmp(part, "elm.text.main.left") == 0)
    {
        return strdup(label_reminder);
    }

    return NULL;
}

static void _reminder_btn_lable_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
    edit_view_data *ad = (edit_view_data *)evas_object_data_get(obj, KEY_PRIORITY_DATA);
    RETM_IF(!ad, "Passed NULL input data");

    evas_object_del(ad->reminder_ctxpopup);
    ad->reminder_ctxpopup = NULL;

    if(!data)
    {
        return;
    }

    char * name = (char *)data;
    calendar_alarm_time_unit_type_e reminder = settings_reminder_type_get(name);
    elm_object_text_set(ad->reminder_btn, name);
    evas_object_size_hint_weight_set(ad->reminder_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(ad->reminder_btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_min_set(ad->reminder_btn, ELM_SCALE_SIZE(BUTTON_SCALE_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));
    ad->reminder_type = reminder;
    ad->reminder_type_name = name;
}

static void _show_reminder_context_popup(edit_view_data *ad)
{
    ad->reminder_ctxpopup = elm_ctxpopup_add(ad->layout);
    evas_object_data_set(ad->reminder_ctxpopup, KEY_PRIORITY_DATA, ad);

    int i;
    for (i = 0; i < reminder_info_size; i++)
    {
        elm_ctxpopup_item_append(ad->reminder_ctxpopup, reminder_info[i].name, NULL, _reminder_btn_lable_changed_cb, (void *)reminder_info[i].name);
    }

    evas_object_size_hint_weight_set(ad->reminder_ctxpopup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

    int x = 0;
    int y = 0;
    evas_object_geometry_get(ad->reminder_btn, &x, &y, NULL, NULL);
    evas_object_move(ad->reminder_ctxpopup, x, y);
    evas_object_smart_callback_add(ad->reminder_ctxpopup, "dismissed",  _reminder_btn_lable_changed_cb, NULL);

    evas_object_show(ad->reminder_ctxpopup);
}

static void _reminder_btn_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
    edit_view_data *ad = (edit_view_data *)data;
    RETM_IF(!ad, "Passed NULL input data");
    if(!ad->reminder_ctxpopup)
    {
       _show_reminder_context_popup(ad);
    }
    else
    {
        evas_object_del(ad->reminder_ctxpopup);
        ad->reminder_ctxpopup = NULL;
    }
}

static void _reminder_lbl_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
    edit_view_data *ad = (edit_view_data *)data;
    ad->reminder = atoi(elm_entry_entry_get(obj));
}

static Evas_Object *_get_content_reminder(void *data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!data, NULL, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;
    Evas_Object *icon = NULL;

    if(strcmp(part, "elm.icon.2") == 0)
    {
        Evas_Object *btn = elm_button_add(view_data->layout);
        get_record_reminder(view_data->event_record, view_data->reminder, &view_data->reminder_type);
        view_data->reminder_type_name = settings_reminder_name_get(view_data->reminder_type, view_data->reminder);
        elm_object_text_set(btn, view_data->reminder_type_name);
        evas_object_smart_callback_add(btn, "clicked", _reminder_btn_clicked_cb, view_data);
        evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_size_hint_align_set(btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
        evas_object_size_hint_min_set(btn, ELM_SCALE_SIZE(BUTTON_SCALE_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));
        view_data->reminder_btn = btn;
        icon = btn;
    }

    return icon;
}

static char *_get_label_recurrence(void *data, Evas_Object *obj, const char *part)
{
    return strdup(label_recurrence);
}

static void _recurrence_btn_lable_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
    edit_view_data *ad = (edit_view_data *)data;
    RETM_IF(!ad, "Passed NULL input data");

    switch(ad->recurrence_data.id)
    {
        case CALENDAR_RECURRENCE_NONE:
            elm_object_text_set(ad->recurrence_btn, "None");
            break;
        case CALENDAR_RECURRENCE_DAILY:
            elm_object_text_set(ad->recurrence_btn, "Daily");
            break;
        case CALENDAR_RECURRENCE_WEEKLY:
            elm_object_text_set(ad->recurrence_btn, "Weekly");
            break;
        case CALENDAR_RECURRENCE_MONTHLY:
            elm_object_text_set(ad->recurrence_btn, "Monthly");
            break;
        case CALENDAR_RECURRENCE_YEARLY:
            elm_object_text_set(ad->recurrence_btn, "Yearly");
            break;
        default:
            break;
    }
    evas_object_size_hint_weight_set(ad->recurrence_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(ad->recurrence_btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_min_set(ad->recurrence_btn, ELM_SCALE_SIZE(BUTTON_SCALE_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));
}

static void _recurrence_btn_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
    edit_view_data *ad = (edit_view_data *)data;
    RETM_IF(!ad, "Passed NULL input data");
    recurrence_view_add(ad->win, ad->navi_frame, &ad->recurrence_data, _recurrence_btn_lable_changed_cb, ad);
}

static const char *_btn_recurrence_text_get(void *data)
{
    RETVM_IF(!data, NULL, "Passed NULL input data");
    edit_view_data *details_data = (edit_view_data *)data;
    int freq = CALENDAR_RECURRENCE_NONE;
    char *id_str = NULL;
    const char *label = NULL;

    if(CALENDAR_ERROR_NONE == calendar_record_get_str(details_data->event_record, _calendar_event.recurrence_id, &id_str))
    {
        if (id_str)
        {
            freq = atoi(id_str);
            free(id_str);
        }
        else
        {
            calendar_record_get_int(details_data->event_record, _calendar_event.freq, &freq);
        }
    }

    details_data->recurrence_data.id = freq;

    int range_type = 0;
    calendar_error_e error = calendar_record_get_int(details_data->event_record, _calendar_event.range_type, &range_type);
    WARN_IF(error != CALENDAR_ERROR_NONE, "calendar_record_get_int() is failed(%x)", error);
    details_data->recurrence_data.range_type = range_type;

    error = calendar_record_get_int(details_data->event_record, _calendar_event.count, &details_data->recurrence_data.count);
    WARN_IF(error != CALENDAR_ERROR_NONE, "calendar_record_set_int() is failed(%x)", error);

    calendar_time_s cal_until_time;
    memset(&cal_until_time, 0x0, sizeof(cal_until_time));
    error = calendar_record_get_caltime(details_data->event_record, _calendar_event.until_time, &cal_until_time);
    WARN_IF(error != CALENDAR_ERROR_NONE, "calendar_record_get_caltime() is failed(%x)", error);
    time_t start_time = cal_until_time.time.utime;
    details_data->recurrence_data.tm = *localtime(&start_time);
    if(cal_until_time.type == CALENDAR_TIME_LOCALTIME)
    {
        details_data->recurrence_data.tm.tm_year = cal_until_time.time.date.year - TM_START_YEAR;
        details_data->recurrence_data.tm.tm_mon = cal_until_time.time.date.month - TM_START_MONTH;
        details_data->recurrence_data.tm.tm_mday = cal_until_time.time.date.mday;
    }

    switch(details_data->recurrence_data.id)
    {
        case CALENDAR_RECURRENCE_DAILY:
            label = "Daily";
            break;
        case CALENDAR_RECURRENCE_WEEKLY:
            label = "Weekly";
            break;
        case CALENDAR_RECURRENCE_MONTHLY:
            label = "Monthly";
            break;
        case CALENDAR_RECURRENCE_YEARLY:
            label = "Yearly";
            break;
        case CALENDAR_RECURRENCE_NONE:
        default:
            label = label_none;
            break;
    }

    return label;
}

static Evas_Object *_get_content_recurrence(void *data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!data, NULL, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    if(strcmp(part, "elm.swallow.end") == 0)
    {
        view_data->recurrence_btn = elm_button_add(view_data->layout);
        evas_object_smart_callback_add(view_data->recurrence_btn, "clicked", NULL, view_data);
        elm_object_text_set(view_data->recurrence_btn, _btn_recurrence_text_get(view_data));
        evas_object_smart_callback_add(view_data->recurrence_btn, "clicked", _recurrence_btn_clicked_cb, data);
        evas_object_size_hint_weight_set(view_data->recurrence_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_size_hint_align_set(view_data->recurrence_btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
        evas_object_size_hint_min_set(view_data->recurrence_btn, ELM_SCALE_SIZE(BUTTON_SCALE_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));
        return view_data->recurrence_btn;
    }
    return NULL;
}

static char *_get_label_priority(void *data, Evas_Object *obj, const char *part)
{
    return strdup(label_priority);
}


static void _set_priority(edit_view_data *view_data, int priority)
{
    RETM_IF(!view_data, "Passed NULL input data");

    switch(priority)
    {
        case CALENDAR_EVENT_PRIORITY_LOW:
            elm_object_text_set(view_data->priority_button, "Low");
            view_data->priority = priority;
            break;
        case CALENDAR_EVENT_PRIORITY_NORMAL:
            elm_object_text_set(view_data->priority_button, "Normal");
            view_data->priority = priority;
            break;
        case CALENDAR_EVENT_PRIORITY_HIGH:
            elm_object_text_set(view_data->priority_button, "High");
            view_data->priority = priority;
            break;
        default:
            ERR("set_priority illegal priority");
            break;
    }
    evas_object_size_hint_weight_set(view_data->priority_button, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(view_data->priority_button, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_min_set(view_data->priority_button, ELM_SCALE_SIZE(BUTTON_SCALE_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));
}

static void _delete_context_popup(edit_view_data *view_data)
{
    if (view_data && view_data->context_popup)
    {
        evas_object_del(view_data->context_popup);
        view_data->context_popup = NULL;
    }
}

static void _set_low_priority_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    _delete_context_popup(view_data);

    _set_priority(view_data, CALENDAR_EVENT_PRIORITY_LOW);
}

static void _set_normal_priority_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    _delete_context_popup(view_data);

    _set_priority(view_data, CALENDAR_EVENT_PRIORITY_NORMAL);
}

static void _set_high_priority_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    _delete_context_popup(view_data);

    _set_priority(view_data, CALENDAR_EVENT_PRIORITY_HIGH);
}

static void _context_popup_dismiss_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    _delete_context_popup(view_data);
}

static void _priority_button_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    _delete_context_popup(view_data);

    view_data->context_popup = elm_ctxpopup_add(view_data->layout);
    elm_ctxpopup_item_append(view_data->context_popup, "Low", NULL, _set_low_priority_cb, view_data);
    elm_ctxpopup_item_append(view_data->context_popup, "Normal", NULL, _set_normal_priority_cb, view_data);
    elm_ctxpopup_item_append(view_data->context_popup, "High", NULL, _set_high_priority_cb, view_data);

    evas_object_size_hint_weight_set(view_data->context_popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

    int x = 0 ,y = 0;
    evas_object_geometry_get(obj, &x, &y, NULL, NULL);
    evas_object_move(view_data->context_popup, x, y);
    evas_object_smart_callback_add(view_data->context_popup, "dismissed", _context_popup_dismiss_cb, view_data);

    evas_object_show(view_data->context_popup);
}

static Evas_Object *_get_content_priority(void *data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!data, NULL, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    if(strcmp(part, "elm.swallow.end") == 0)
    {
        view_data->priority_button = elm_button_add(view_data->layout);
        evas_object_smart_callback_add(view_data->priority_button, "clicked", _priority_button_cb, view_data);

        int priority = 0;
        calendar_record_get_int(view_data->event_record, _calendar_event.priority, &priority);
        _set_priority(view_data, priority);
        return view_data->priority_button;
    }

    return NULL;
}

static char *_get_label_sensitivity(void *data, Evas_Object *obj, const char *part)
{
    return strdup(label_sensitivity);
}

static void _set_sensitivity(edit_view_data *view_data, int sensitivity)
{
    RETM_IF(!view_data, "Passed NULL input data");

    switch(sensitivity)
    {
        case CALENDAR_SENSITIVITY_PUBLIC:
            elm_object_text_set(view_data->sensitivity_button, "Public");
            view_data->sensitivity = sensitivity;
            break;
        case CALENDAR_SENSITIVITY_PRIVATE:
            elm_object_text_set(view_data->sensitivity_button, "Private");
            view_data->sensitivity = sensitivity;
            break;
        case CALENDAR_SENSITIVITY_CONFIDENTIAL:
            elm_object_text_set(view_data->sensitivity_button, "Confidential");
            view_data->sensitivity = sensitivity;
            break;
        default:
            ERR("set_sensitivity illegal priority");
            break;
    }
    evas_object_size_hint_weight_set(view_data->sensitivity_button, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(view_data->sensitivity_button, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_min_set(view_data->sensitivity_button, ELM_SCALE_SIZE(BUTTON_SCALE_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));
}

static void _set_public_sensitivity_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    _delete_context_popup(view_data);

    _set_sensitivity(view_data, CALENDAR_SENSITIVITY_PUBLIC);
}

static void _set_private_sensitivity_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    _delete_context_popup(view_data);

    _set_sensitivity(view_data, CALENDAR_SENSITIVITY_PRIVATE);
}


static void _set_confidential_sensitivity_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    _delete_context_popup(view_data);

    _set_sensitivity(view_data, CALENDAR_SENSITIVITY_CONFIDENTIAL);
}

static void _sensitivity_button_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    _delete_context_popup(view_data);

    view_data->context_popup = elm_ctxpopup_add(view_data->layout);
    elm_ctxpopup_item_append(view_data->context_popup, "Public", NULL, _set_public_sensitivity_cb, view_data);
    elm_ctxpopup_item_append(view_data->context_popup, "Private", NULL, _set_private_sensitivity_cb, view_data);
    elm_ctxpopup_item_append(view_data->context_popup, "Confidential", NULL, _set_confidential_sensitivity_cb, view_data);

    evas_object_size_hint_weight_set(view_data->context_popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

    int x = 0 ,y = 0;
    evas_object_geometry_get(obj, &x, &y, NULL, NULL);
    evas_object_move(view_data->context_popup, x, y);
    evas_object_smart_callback_add(view_data->context_popup, "dismissed", _context_popup_dismiss_cb, view_data);

    evas_object_show(view_data->context_popup);
}

static Evas_Object *_get_content_sensivity(void *data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!data, NULL, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    if(strcmp(part, "elm.swallow.end") == 0)
    {
        view_data->sensitivity_button = elm_button_add(view_data->layout);
        evas_object_smart_callback_add(view_data->sensitivity_button, "clicked", _sensitivity_button_cb, view_data);
        elm_object_text_set(view_data->sensitivity_button, "Public");

        int sensitivity = 0;
        calendar_record_get_int(view_data->event_record, _calendar_event.sensitivity, &sensitivity);
        _set_sensitivity(view_data, sensitivity);

        return view_data->sensitivity_button;
    }

    return NULL;
}

static char *_get_label_status(void *data, Evas_Object *obj, const char *part)
{
    return strdup(label_status);
}

static void _set_status(edit_view_data *view_data, int status)
{
    RETM_IF(!view_data, "Passed NULL input data");

    elm_object_text_set(view_data->status_button, settings_status_name_get(status));
    evas_object_size_hint_weight_set(view_data->status_button, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(view_data->status_button, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_min_set(view_data->status_button, ELM_SCALE_SIZE(BUTTON_SCALE_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));
    view_data->status = status;
}

static void _set_none_status_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    _delete_context_popup(view_data);

    _set_status(view_data, CALENDAR_EVENT_STATUS_NONE);
}

static void _set_confirmed_status_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    _delete_context_popup(view_data);

    _set_status(view_data, CALENDAR_EVENT_STATUS_CONFIRMED);
}

static void _set_cancelled_status_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    _delete_context_popup(view_data);

    _set_status(view_data, CALENDAR_EVENT_STATUS_CANCELLED);
}

static void _set_tentative_status_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    _delete_context_popup(view_data);

    _set_status(view_data, CALENDAR_EVENT_STATUS_TENTATIVE);
}

static void _status_button_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    _delete_context_popup(view_data);

    view_data->context_popup = elm_ctxpopup_add(view_data->layout);
    elm_ctxpopup_item_append(view_data->context_popup, "None", NULL, _set_none_status_cb, view_data);
    elm_ctxpopup_item_append(view_data->context_popup, "Confirmed", NULL, _set_confirmed_status_cb, view_data);
    elm_ctxpopup_item_append(view_data->context_popup, "Cancelled", NULL, _set_cancelled_status_cb, view_data);
    elm_ctxpopup_item_append(view_data->context_popup, "Tentative", NULL, _set_tentative_status_cb, view_data);

    evas_object_size_hint_weight_set(view_data->context_popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

    int x = 0 ,y = 0;
    evas_object_geometry_get(obj, &x, &y, NULL, NULL);
    evas_object_move(view_data->context_popup, x, y);
    evas_object_smart_callback_add(view_data->context_popup, "dismissed", _context_popup_dismiss_cb, view_data);

    evas_object_show(view_data->context_popup);
}

static Evas_Object *_get_content_status(void *data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!data, NULL, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data  *)data;

    if(strcmp(part, "elm.swallow.end") == 0)
    {
        view_data->status_button = elm_button_add(view_data->layout);
        evas_object_smart_callback_add(view_data->status_button, "clicked", _status_button_cb, view_data);

        int status = 0;
        calendar_record_get_int(view_data->event_record, _calendar_event.event_status, &status);
        _set_status(view_data, status);
        return view_data->status_button;
    }

    return NULL;
}

static Elm_Genlist_Item_Class itc_title = {
    .item_style = "1icon",
    .func.content_get = _get_title_content,
};

static Elm_Genlist_Item_Class itc_from = {
    .item_style = "default",
    .func.text_get = _get_label_from,
    .func.content_get = _get_content_from,
};

static Elm_Genlist_Item_Class itc_to = {
    .item_style = "default",
    .func.text_get = _get_label_to,
    .func.content_get = _get_content_to,
};

static Elm_Genlist_Item_Class itc_timezone = {
    .item_style = "default",
    .func.text_get = _label_timezone_get,
    .func.content_get = _get_content_timezone,
};

static Elm_Genlist_Item_Class itc_allday = {
    .item_style = "default",
    .func.text_get = _get_label_allday,
    .func.content_get = _get_content_allday,
};

static Elm_Genlist_Item_Class itc_location = {
    .item_style = "1icon",
    .func.content_get = _get_content_location,
};

static Elm_Genlist_Item_Class itc_description = {
    .item_style = "1icon",
    .func.content_get = _get_content_description,
};

static Elm_Genlist_Item_Class itc_reminder = {
    .item_style = "1line",
    .func.text_get = _get_label_reminder,
    .func.content_get = _get_content_reminder,
};

static Elm_Genlist_Item_Class itc_recurrence = {
    .item_style = "default",
    .func.text_get = _get_label_recurrence,
    .func.content_get = _get_content_recurrence,
};

static Elm_Genlist_Item_Class itc_priority = {
    .item_style = "default",
    .func.text_get = _get_label_priority,
    .func.content_get = _get_content_priority,
};

static Elm_Genlist_Item_Class itc_sensivity = {
    .item_style = "default",
    .func.text_get = _get_label_sensitivity,
    .func.content_get = _get_content_sensivity,
};

static Elm_Genlist_Item_Class itc_status = {
    .item_style = "default",
    .func.text_get = _get_label_status,
    .func.content_get = _get_content_status,
};

static void genlist_items_append(edit_view_data *data)
{
    elm_genlist_item_append(data->genlist, &itc_title, data, NULL, ELM_GENLIST_ITEM_NONE, NULL, data);

    data->genlist_from_item =
    elm_genlist_item_append(data->genlist, &itc_from, data, NULL, ELM_GENLIST_ITEM_NONE, NULL, data);

    data->genlist_to_item =
    elm_genlist_item_append(data->genlist, &itc_to, data, NULL, ELM_GENLIST_ITEM_NONE, NULL, data);

    elm_genlist_item_append(data->genlist, &itc_timezone, data, NULL, ELM_GENLIST_ITEM_NONE, NULL, data);

    elm_genlist_item_append(data->genlist, &itc_allday, data, NULL, ELM_GENLIST_ITEM_NONE, NULL, data);

    data->genlist_location_item =
    elm_genlist_item_append(data->genlist, &itc_location, data, NULL, ELM_GENLIST_ITEM_NONE, NULL, data);

    data->genlist_description_item =
    elm_genlist_item_append(data->genlist, &itc_description, data, NULL, ELM_GENLIST_ITEM_NONE, NULL, data);

    data->genlist_reminder_item =
    elm_genlist_item_append(data->genlist, &itc_reminder, data, NULL, ELM_GENLIST_ITEM_NONE, NULL, data);

    elm_genlist_item_append(data->genlist, &itc_recurrence, data, NULL, ELM_GENLIST_ITEM_NONE, NULL, data);

    elm_genlist_item_append(data->genlist, &itc_priority, data, NULL, ELM_GENLIST_ITEM_NONE, NULL, data);

    elm_genlist_item_append(data->genlist, &itc_sensivity, data, NULL, ELM_GENLIST_ITEM_NONE, NULL, data);

    elm_genlist_item_append(data->genlist, &itc_status, data, NULL, ELM_GENLIST_ITEM_NONE, NULL, data);

    evas_object_smart_callback_add(data->genlist, "selected", _scroll_genlist_top_cb, data);
}

static bool _is_title_valid(const char * title)
{
    RETVM_IF(!title, false, "passed string is NULL");

    if (strlen(title) == 0)
    {
        return false;
    }

    while(isspace(*title))
    {
        ++title;
    }

    return *title != '\0';
}

static bool _is_datetime_valid(edit_view_data *ad)
{
    if (difftime(mktime(&ad->etm), mktime(&ad->stm)) < 0)
    {
        return false;
    }
    return true;
}

static void _popup_button_ok_cb(void *data, Evas_Object *obj, void *event_info)
{
    edit_view_data *ad = (edit_view_data *)data;
    RETM_IF(!ad, "Data NULL");
    evas_object_del(ad->popup);
    ad->popup = NULL;
}

static Evas_Object *_create_warning_popup(void *data, const char *popup_text)
{
    edit_view_data *ad = (edit_view_data *)data;
    RETVM_IF(!ad, NULL, "Passed NULL input data");

    Evas_Object *popup = elm_popup_add(ad->navi_frame);
    RETVM_IF(!popup, NULL, "Can't create popup");

    Evas_Object *popup_content = elm_label_add(popup);
    elm_object_text_set(popup_content, popup_text);
    elm_object_content_set(popup, popup_content);

    Evas_Object * button_ok = elm_button_add(popup);
    RETVM_IF(!button_ok, NULL, "Cannot add button");
    elm_object_text_set(button_ok, "OK");
    elm_object_part_content_set(popup, "button1", button_ok);
    evas_object_smart_callback_add(button_ok, "clicked", _popup_button_ok_cb, ad);

    evas_object_show(popup);

    return popup;
}

static void _save_button_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    edit_view_data *view_data = (edit_view_data *)data;

    elm_toolbar_item_selected_set(event_info, EINA_FALSE);

    if (!_is_title_valid(view_data->title))
    {
        view_data->popup = _create_warning_popup(view_data, WARNING_POPUP_TEXT);
        return;
    }

    if (!_is_datetime_valid(view_data))
    {
        view_data->popup = _create_warning_popup(view_data, WARNING_TIME_POPUP_TEXT);
        return;
    }

    calendar_error_e error = calendar_connect();
    RETM_IF(error != CALENDAR_ERROR_NONE, "calendar_connect() is failed(%x)", error);
    set_record_title(view_data->event_record, view_data->title);

    bool is_all_day = elm_check_state_get(view_data->check_object);
    set_record_time(&view_data->stm, &view_data->etm, view_data->event_record, is_all_day);

    set_record_location(view_data->event_record, view_data->location, CALENDAR_RECORD_NO_COORDINATE, CALENDAR_RECORD_NO_COORDINATE);

    set_record_note(view_data->event_record, view_data->description);

    set_record_priority(view_data->event_record, view_data->priority);

    set_record_sensitivity(view_data->event_record, view_data->sensitivity);

    set_record_status(view_data->event_record, view_data->status);

    set_record_reminder(view_data->event_record, view_data->reminder_type_name);

    set_record_recurrence(view_data->event_record, view_data->recurrence_data.id);

    set_record_repeat_range_type(view_data->event_record, view_data->recurrence_data.range_type);

    set_record_repeat_count(view_data->event_record, view_data->recurrence_data.count);

    set_record_repeat_until_time(view_data->event_record, &view_data->recurrence_data.tm);

    error = calendar_db_update_record(view_data->event_record);
    WARN_IF(error != CALENDAR_ERROR_NONE, "calendar_db_update_record() is failed(%x)", error);

    error = calendar_disconnect();
    WARN_IF(error != CALENDAR_ERROR_NONE, "calendar_disconnect() is failed(%x)", error);

    elm_naviframe_item_pop(view_data->navi_frame);
}

static Evas_Object *_create_edit_view_layout(Evas_Object * navi_frame)
{
    RETVM_IF(!navi_frame, NULL, "navi_frame NULL");

    Evas_Object *layout = elm_layout_add(navi_frame);
    elm_layout_theme_set(layout, "layout", "application", "default");
    evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(layout, EVAS_HINT_FILL, EVAS_HINT_FILL);
    elm_layout_file_set(layout, GET_PATH(EDJ_CREATEVIEW), "create_view_layout");
    evas_object_show(layout);

    return layout;
}

static Evas_Object *_create_edit_view_genlist(Evas_Object * layout)
{
    RETVM_IF(!layout, NULL, "layout NULL");

    Evas_Object *genlist = elm_genlist_add(layout);
    elm_genlist_select_mode_set(genlist, ELM_OBJECT_SELECT_MODE_ALWAYS);
    evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);
    elm_object_part_content_set(layout, "elm.swallow.genlist.layout", genlist);
    evas_object_show(genlist);

    return genlist;
}

static bool _close_active_popup(edit_view_data *view_data)
{
    RETVM_IF(!view_data, false, "view_data is NULL");

    if (view_data->context_popup)
    {
        evas_object_del(view_data->context_popup);
        view_data->context_popup = NULL;
        return true;
    }
    else if (view_data->popup)
    {
        evas_object_del(view_data->popup);
        view_data->popup = NULL;
        return true;
    }
    else if (view_data->date_time_popup)
    {
        Evas_Object *obj = elm_object_content_get(view_data->date_time_popup);
        elm_object_content_unset(view_data->date_time_popup);
        evas_object_hide(obj);
        evas_object_del(view_data->date_time_popup);
        view_data->date_time_popup = NULL;
        return true;
    }
    else if (view_data->reminder_ctxpopup)
    {
        evas_object_del(view_data->reminder_ctxpopup);
        view_data->reminder_ctxpopup = NULL;
        return true;
    }

    return false;
}

static Eina_Bool _edit_view_navi_item_cb(void *data, Elm_Object_Item *it)
{
    RETVM_IF(!data, false, "view_data is NULL");
    edit_view_data *view_data = (edit_view_data *)data;

    if (_close_active_popup(view_data))
    {
        return EINA_FALSE;
    }
    else
    {
        evas_object_del(view_data->layout);

        calendar_record_destroy(view_data->event_record, EINA_TRUE);

        evas_object_del(view_data->start_datetime);
        evas_object_del(view_data->end_datetime);
        free(view_data->title);
        free(view_data->description);
        free(view_data->location);

        free(view_data);

        return EINA_TRUE;
    }
}

Evas_Object *open_edit_view(window_obj *win, Evas_Object *navi_frame, int index)
{
    RETVM_IF(!navi_frame, NULL, "navi_frame NULL");

    calendar_error_e error = calendar_connect();
    RETVM_IF(error != CALENDAR_ERROR_NONE, NULL, "calendar_connect() is failed(%x)", error);

    edit_view_data *view_data = calloc(1, sizeof(edit_view_data));
    view_data->navi_frame = navi_frame;
    view_data->win = win;

    view_data->layout = _create_edit_view_layout(view_data->navi_frame);

    view_data->genlist = _create_edit_view_genlist(view_data->layout);

    calendar_db_get_record(_calendar_event._uri, index, &view_data->event_record);

    get_record_time(view_data->event_record, &view_data->stm, &view_data->etm);

    genlist_items_append(view_data);

    error = calendar_disconnect();
    WARN_IF(error != CALENDAR_ERROR_NONE, "calendar_disconnect() is failed(%x)", error);

    view_data->naviframe_item = elm_naviframe_item_push(view_data->navi_frame, "Edit event", NULL, NULL, view_data->layout, NULL);
    elm_naviframe_item_pop_cb_set(view_data->naviframe_item, _edit_view_navi_item_cb, view_data);

    Evas_Object * toolbar = ui_utils_toolbar_add(view_data->navi_frame, view_data->naviframe_item);
    elm_toolbar_item_append(toolbar, NULL, save_button, _save_button_clicked_cb, view_data);

    return view_data->layout;
}
